﻿using System;
using System.IO;
using System.Text.RegularExpressions;

namespace rxn
{
    internal class Program
    {
        /// <summary>
        /// Performs regex renaming of files with optional preview
        /// </summary>
        /// <param name="args"></param>
        private static void Main(string[] args)
        {
            RegexOptions rxOptions = RegexOptions.None;
            //bool recurseDirectories = false;
            //bool renameDirectories = false;
            bool preview = false;
            string options = null;

            if (args.Length < 2)
            {
                Console.WriteLine("MISSING ARGS");
                Usage();
                return;
            }
            string replacement = args[1];
            if (args.Length > 2)
            {
                options = args[2].ToLower();
                if (options.StartsWith("/"))
                {
                    if (options.IndexOf("c") == -1)
                    {
                        // c not IgnoreCase (case sensitive)
                        rxOptions = RegexOptions.IgnoreCase;
                    }

                    // w IgnorePatternWhitespace
                    if (options.IndexOf("w") > -1)
                    {
                        rxOptions = rxOptions | RegexOptions.IgnorePatternWhitespace;
                    }

                    // r RightToLeft
                    if (options.IndexOf("r") > -1)
                    {
                        rxOptions = rxOptions | RegexOptions.RightToLeft;
                    }

                    //recurseDirectories = options.IndexOf("s") > -1;
                    //renameDirectories = options.IndexOf("d") > -1;
                    preview = options.IndexOf("p") > -1;
                }
                else
                {
                    Console.WriteLine("BAD OPTIONS");
                    Usage();
                    return;
                }
            }

            string pattern = args[0];
            Regex rx = new Regex(pattern, rxOptions);

            string[] files = Directory.GetFiles(Environment.CurrentDirectory);
            foreach (string file in files)
            {
                if (rx.IsMatch(file))
                {
                    string newName = rx.Replace(file, replacement);
                    if (preview)
                    {
                        Console.WriteLine("preview: {0}", newName);
                    }
                    else
                    {
                        try
                        {
                            File.Move(file, newName);
                            Console.WriteLine(newName);
                        }
                        catch (Exception ex)
                        {
                            Console.WriteLine("error : {0}=>{1} {2}", file, newName, ex.Message);
                        }
                    }
                }
                else
                {
                    if (preview)
                    {
                        Console.WriteLine("preview: {0}", file);
                    }
                    else
                    {
                        Console.WriteLine(file);
                    }
                }
            }
        }
        static void Usage()
        {
            Console.WriteLine("RXN.exe - performs regular expression renaming of files.");
            Console.WriteLine("rxn pattern replacement [/pcer]");
            Console.WriteLine("");
            Console.WriteLine(" p Preview changes");
            Console.WriteLine(" c Case sensitive");
            Console.WriteLine(" w Ignore pattern whitespace");
            Console.WriteLine(" r Right-to-left");
            Console.WriteLine("");
        }
    }
}