﻿// <copyright project="Salient.SqlServer.NHibernate" file="SqlGeographyUserType.cs" company="Sky Sanders">
// This source is a Public Domain Dedication.
// Please see http://spikes.codeplex.com/ for details.   
// Attribution is appreciated
// </copyright> 
// <version>1.0</version>
using System;
using System.Data;
using System.IO;
using Microsoft.SqlServer.Types;
using NHibernate;
using NHibernate.SqlTypes;
using NHibernate.UserTypes;

namespace Salient.SqlServer.NHibernate
{
    public class SqlGeographyUserType : IUserType
    {
        #region IUserType Members

        public object NullSafeGet(IDataReader rs, string[] names, object owner)
        {
            if (names.Length != 1)
                throw new InvalidOperationException("names array has more than one element. can't handle this!");

            object obj = NHibernateUtil.BinaryBlob.NullSafeGet(rs, names[0]);
            if (obj == null) return null;
            var bytes = obj as byte[];

            if (bytes == null
                /*i know bytes are not null, but i don't like ignoring blue squigglies */
                || bytes.Length == 0)
            {
                return SqlGeography.Null;
            }

            var returnValue = new SqlGeography();

            using (var input = new MemoryStream(bytes))
            {
                using (var r = new BinaryReader(input))
                {
                    returnValue.Read(r);
                }
            }

            return returnValue;
        }


        public void NullSafeSet(IDbCommand cmd, object value, int index)
        {
            var parameter = (IDataParameter) cmd.Parameters[index];
            if (value == null)
                parameter.Value = DBNull.Value;
            else
            {
                var obj = (SqlGeography) value;
                using (var output = new MemoryStream())
                {
                    using (var w = new BinaryWriter(output))
                    {
                        obj.Write(w);
                    }
                    parameter.Value = output.ToArray();
                }
            }
        }

        public new bool Equals(object x, object y)
        {
            if (x == null || y == null)
                return false;

            var xx = (SqlGeography) x;
            var yy = (SqlGeography) y;
            return yy.ToString() == xx.ToString();
        }

        public int GetHashCode(object x)
        {
            return x.GetHashCode();
        }


        /// <summary>
        /// 
        /// </summary>
        /// <param name="value"></param>
        /// <returns></returns>
        public object DeepCopy(object value)
        {
            //SqlHierarchyId is struct so can just copy it

            var toCopy = (SqlGeography) value;

            if (toCopy == SqlGeography.Null)
                return SqlGeography.Null;
            var copy = new SqlGeography();

            using (var s = new MemoryStream())
            {
                var w = new BinaryWriter(s);
                toCopy.Write(w);
                s.Seek(0, SeekOrigin.Begin);
                var r = new BinaryReader(s);
                copy.Read(r);
            }

            return copy;
        }

        public object Replace(object original, object target, object owner)
        {
            throw new NotImplementedException();
        }

        public object Assemble(object cached, object owner)
        {
            return DeepCopy(cached);
        }

        public object Disassemble(object value)
        {
            return DeepCopy(value);
        }

        public SqlType[] SqlTypes
        {
            get { return new[] {NHibernateUtil.BinaryBlob.SqlType}; }
        }

        public Type ReturnedType
        {
            get { return typeof (SqlGeography); }
        }

        public bool IsMutable
        {
            get { return true; }
        }

        #endregion
    }
}