﻿using System;
using System.Collections;
using System.Configuration;
using System.Drawing;
using System.Text;
using System.Web;
using System.Web.Configuration;
using System.Web.Hosting;
using System.Web.UI;
using System.Web.UI.WebControls;

namespace Salient.Web.Administration.Security.Permissions
{
    public partial class managePermissions : SecurityPage
    {
        private const string NO_DELETE_RULES = "WebAdminNoDeleteRules";
        private const string PARENT_RULE_COUNT = "WebAdminParentRuleCount";
        private const string RULES = "WebAdminRules";
        private const string SELECTED_ITEM = "WebAdminSelectedItem";
        private const string SELECTED_RULE = "WebAdminSelectedRule";

        private int ParentRuleCount
        {
            get
            {
                object obj = Session[PARENT_RULE_COUNT];
                return obj != null ? (int) obj : -1;
            }
            set { Session[PARENT_RULE_COUNT] = value; }
        }

        private ArrayList NotDeleteableRules
        {
            get
            {
                object obj = Session[NO_DELETE_RULES];
                if (obj != null)
                {
                    return (ArrayList) Session[NO_DELETE_RULES];
                }
                else
                {
                    return (new ArrayList());
                }
            }
            set { Session[NO_DELETE_RULES] = value; }
        }

        private ArrayList Rules
        {
            get
            {
                object obj = Session[RULES];
                if (obj != null)
                {
                    return (ArrayList) Session[RULES];
                }
                else
                {
                    return (new ArrayList());
                }
            }
            set { Session[RULES] = value; }
        }

        private int SelectedRule
        {
            get
            {
                object obj = Session[SELECTED_RULE];
                return obj != null ? (int) obj : -1;
            }
            set { Session[SELECTED_RULE] = value; }
        }

        private void GetNotDeletableRules(System.Configuration.Configuration config)
        {
            var notDeleteableAuth = (AuthorizationSection) config.GetSection("system.web/authorization");
            var arrNoDelete = new ArrayList();
            PropertyInformation propUsers = null;
            PropertyInformation propRoles = null;
            bool entryIsDeletable = false;
            int i = 0;
            foreach (AuthorizationRule entryKeep in notDeleteableAuth.Rules)
            {
                entryIsDeletable = true;
                propUsers = entryKeep.ElementInformation.Properties["users"];
                propRoles = entryKeep.ElementInformation.Properties["roles"];

                if (propUsers != null)
                {
                    if (propUsers.ValueOrigin == PropertyValueOrigin.Inherited)
                    {
                        entryIsDeletable = false;
                    }
                }

                if (propRoles != null && entryIsDeletable)
                {
                    if (propRoles.ValueOrigin == PropertyValueOrigin.Inherited)
                    {
                        entryIsDeletable = false;
                    }
                }

                if (!entryIsDeletable)
                {
                    // store the index in here as to which one is not deletable
                    arrNoDelete.Add(i);
                }

                i++;
            }

            Session[NO_DELETE_RULES] = arrNoDelete;
        }

        protected string GetToolTip(string resourceName, string itemName)
        {
            var tempString = (string) GetLocalResourceObject(resourceName);
            return String.Format((string) GetGlobalResourceObject("GlobalResources", "ToolTipFormat"), tempString,
                                 itemName);
        }

        private void BindGrid()
        {
            string appPath = CurrentPath;
            string parentPath = GetParentPath(appPath);

            System.Configuration.Configuration config = OpenWebConfiguration(appPath, true);
            var auth = (AuthorizationSection) config.GetSection("system.web/authorization");

            System.Configuration.Configuration parentConfig = OpenWebConfiguration(parentPath, true);
            var parentAuth = (AuthorizationSection) parentConfig.GetSection("system.web/authorization");
            ParentRuleCount = parentAuth.Rules.Count;

            GetNotDeletableRules(config);

            var arr = new ArrayList();
            foreach (AuthorizationRule entry in auth.Rules)
            {
                arr.Add(entry);
            }

            Session[RULES] = arr;
            dataGrid.DataSource = arr;
            dataGrid.DataBind();
            if (dataGrid.SelectedRow != null)
            {
                UpdateRowColors(dataGrid, dataGrid.Rows[dataGrid.SelectedRow.RowIndex]);
            }
        }

        protected void DeleteRule(object sender, EventArgs e)
        {
            var button = (LinkButton) sender;
            var item = (GridViewRow) button.Parent.Parent;
            var rule = (AuthorizationRule) Rules[item.RowIndex];
            var builder = new StringBuilder();
            builder.Append(rule.Action);
            int i = 0;
            foreach (string u in rule.Users)
            {
                if (i > 0)
                {
                    builder.Append(", " + u);
                }
                else
                {
                    builder.Append(" " + u);
                }
                i++;
            }
            i = 0;
            foreach (string r in rule.Roles)
            {
                if (i > 0)
                {
                    builder.Append(", " + r);
                }
                else
                {
                    builder.Append(" " + r);
                }
                i++;
            }
            RuleDescription.Text = builder.ToString();
            ((WebAdminWithConfirmation) Master).SetDisplayUI(true);
            Session["ItemIndex"] = item.RowIndex;
        }

        protected void Yes_Click(object sender, EventArgs e)
        {
            Rules.RemoveAt((int) Session["ItemIndex"]);
            UpdateRules();
            dataGrid.SelectedIndex = -1;
            BindGrid();
            ((WebAdminWithConfirmation) Master).SetDisplayUI(false);
        }

        protected void No_Click(object sender, EventArgs e)
        {
            ((WebAdminWithConfirmation) Master).SetDisplayUI(false);
        }


        private string GetRoles(object val, bool appendImg)
        {
            var builder = new StringBuilder();
            var rule = (AuthorizationRule) val;
            if (rule.Roles.Count == 0)
            {
                return String.Empty;
            }
            for (int i = 0; i < rule.Roles.Count; i++)
            {
                if (i > 0)
                {
                    builder.Append(", ");
                }
                string role = rule.Roles[i];
                if (role == "*")
                {
                    role = (string) GetLocalResourceObject("BracketAll");
                }
                builder.Append(role);
            }
            if (appendImg)
            {
                var builder2 = new StringBuilder();
                builder2.Append("<img src=\"../../Images/image2.gif\" alt=\"" +
                                (string) GetGlobalResourceObject("GlobalResources", "RoleGif") + " [" + builder + "]" +
                                "\"/> ");
                builder2.Append(builder.ToString());
                return builder2.ToString();
            }
            else
            {
                return builder.ToString();
            }
        }


        private string GetUsers(object val, bool appendImg)
        {
            var builder = new StringBuilder();
            var rule = (AuthorizationRule) val;
            if (rule.Users.Count == 0)
            {
                return String.Empty;
            }
            for (int i = 0; i < rule.Users.Count; i++)
            {
                if (i > 0)
                {
                    builder.Append(", ");
                }
                string user = rule.Users[i];
                if (user == "?")
                {
                    user = (string) GetLocalResourceObject("BracketAnonymous");
                }
                else if (user == "*")
                {
                    user = (string) GetLocalResourceObject("BracketAll");
                }
                builder.Append(user);
            }
            if (appendImg)
            {
                var builder2 = new StringBuilder();
                builder2.Append("<img src=\"../../Images/image1.gif\" alt=\"" +
                                (string) GetGlobalResourceObject("GlobalResources", "UserGif") + " [" + builder + "]" +
                                "\"/> ");
                builder2.Append(builder.ToString());
                return builder2.ToString();
            }
            else
            {
                return builder.ToString();
            }
        }


        protected string GetAction(object val)
        {
            var rule = (AuthorizationRule) val;
            string ruleAction = "";
            if (rule.Action == AuthorizationRuleAction.Allow)
            {
                ruleAction = (string) GetLocalResourceObject("Allow");
            }
            else if (rule.Action == AuthorizationRuleAction.Deny)
            {
                ruleAction = (string) GetLocalResourceObject("Deny");
            }
            return ruleAction;
        }

        protected string GetUsersAndRoles(object val, bool appendImg)
        {
            return GetUsers(val, appendImg) + GetRoles(val, appendImg);
        }


        protected bool IsEntryDeleteable(int rowIndex)
        {
            bool entryIsDeleteable = false;
            if (rowIndex < Rules.Count - ParentRuleCount)
            {
                entryIsDeleteable = true;
            }
            if (!entryIsDeleteable)
            {
                return entryIsDeleteable;
            }
            foreach (int index1 in NotDeleteableRules)
            {
                if (index1 == rowIndex)
                {
                    entryIsDeleteable = false;
                }
            }
            return entryIsDeleteable;
        }

        private bool IsIE()
        {
            HttpBrowserCapabilities caps = Page.Request.Browser;
            bool isIE = (caps.Type.IndexOf("IE") > -1);
            return isIE;
        }

        protected void ItemDataBound(object sender, GridViewRowEventArgs e)
        {
            if (e.Row.RowIndex >= Rules.Count - ParentRuleCount)
            {
                return;
            }
            DataControlRowType itemType = e.Row.RowType;
            if ((itemType == DataControlRowType.Pager) ||
                (itemType == DataControlRowType.Header) ||
                (itemType == DataControlRowType.Footer))
            {
                return;
            }

            if (IsIE())
            {
                if (IsEntryDeleteable(e.Row.RowIndex))
                {
                    // if netscape, then selecting the the row
                    // will not allow the DeleteRule to fire
                    foreach (Control c in e.Row.Cells[0].Controls)
                    {
                        var button = c as LinkButton;
                        if (button == null)
                        {
                            continue;
                        }
                        e.Row.Attributes["onclick"] = Page.GetPostBackClientHyperlink(button, "");
                    }
                }
            }
        }

        protected void MoveRuleDown(object sender, EventArgs e)
        {
            ArrayList rules = Rules;
            int selectedRule = SelectedRule;
            // // Response.Write(selectedRule);

            if (selectedRule >= rules.Count - ParentRuleCount)
            {
                return;
            }

            var rule = (AuthorizationRule) rules[selectedRule];
            rules.RemoveAt(selectedRule);
            rules.Insert(selectedRule + 1, rule);
            UpdateRules();
            BindGrid();
            dataGrid.SelectedIndex = selectedRule + 1;
            UpdateRowColors(dataGrid, dataGrid.Rows[selectedRule + 1]);
            SelectedRule = selectedRule + 1;
            UpdateUpDownButtons();
        }

        protected void MoveRuleUp(object sender, EventArgs e)
        {
            ArrayList rules = Rules;
            int selectedRule = SelectedRule;

            if (selectedRule == 0)
            {
                return;
            }

            var rule = (AuthorizationRule) rules[selectedRule];
            rules.RemoveAt(selectedRule);
            rules.Insert(selectedRule - 1, rule);
            UpdateRules();
            BindGrid();
            dataGrid.SelectedIndex = selectedRule - 1;
            UpdateRowColors(dataGrid, dataGrid.Rows[selectedRule - 1]);
            SelectedRule = selectedRule - 1;
            UpdateUpDownButtons();
        }

        private string GetDirectory(string path)
        {
            if (path == null)
            {
                return null;
            }

            if (path.LastIndexOf('/') == -1)
            {
                return "/";
            }

            return path.Substring(path.LastIndexOf('/') + 1);
        }

        private void Page_Init()
        {
            if (!IsPostBack)
            {
                // Note: treenodes persist when added in Init, before loadViewState
                var n = new TreeNode(GetDirectory(ApplicationPath), ApplicationPath);
                tv.Nodes.Add(n);
                n.Selected = true;

                if (String.IsNullOrEmpty(CurrentPath))
                {
                    CurrentPath = ApplicationPath;
                }

                PopulateChildren(n, CurrentPath);
            }
            dataGrid.HeaderStyle.HorizontalAlign = DirectionalityHorizontalAlign;
        }

        private void Page_Load()
        {
            if (!IsPostBack)
            {
                BindGrid();
            }

            HighlightSelectedNode(tv.Nodes[0], CurrentPath);
        }

        private void PopulateChildren(TreeNode parent, string selectNodeValue)
        {
            if (parent.ChildNodes.Count == 0)
            {
                VirtualDirectory vdir = GetVirtualDirectory(parent.Value);
                foreach (VirtualDirectory childVdir in vdir.Directories)
                {
                    string childValue = parent.Value + "/" + childVdir.Name;
                    var newNode = new TreeNode(childVdir.Name, childValue);
                    if (selectNodeValue != null && childValue == selectNodeValue)
                    {
                        //newNode.Selected = true;
                        newNode.SelectAction = TreeNodeSelectAction.Select;
                    }
                    parent.ChildNodes.Add(newNode);
                }
            }
        }

        protected bool HighlightSelectedNode(TreeNode node, string path)
        {
            bool foundIt = false;
            foreach (TreeNode childNode in node.ChildNodes)
            {
                if (childNode.Value == path)
                {
                    childNode.Selected = true;
                    foundIt = true;
                    break;
                }
            }
            return foundIt;
        }

        protected void RedirectToCreatePermission(object sender, EventArgs e)
        {
            Response.Redirect("createPermission.aspx");
        }

        public void SelectClick(object sender, EventArgs e)
        {
            var button = (LinkButton) sender;

            var row = (GridViewRow) button.Parent.Parent;
            SelectedRule = row.RowIndex;
            dataGrid.SelectedIndex = SelectedRule;

            UpdateRowColors(dataGrid, row);
            UpdateUpDownButtons();
        }

        private void SetItemColorRecursive(Control c, Color col)
        {
            foreach (Control child in c.Controls)
            {
                var button = child as LinkButton;
                if (button != null)
                {
                    button.ForeColor = col;
                }
                var label = child as Label;
                if (label != null)
                {
                    label.ForeColor = col;
                }
                SetItemColorRecursive(child, col);
            }
        }

        protected void TreeNodeExpanded(Object sender, TreeNodeEventArgs e)
        {
            foreach (TreeNode child in e.Node.ChildNodes)
            {
                PopulateChildren(child, CurrentPath);
            }
        }

        protected void TreeNodeSelected(object sender, EventArgs e)
        {
            CurrentPath = ((TreeView) sender).SelectedNode.Value;
            dataGrid.SelectedIndex = -1;
            BindGrid();
        }

        public void UpdateRowColors(Control dataGrid, Control item)
        {
            if (item == null)
            {
                // unexpected condition -exit gracefully
                return;
            }

            var prevItemID = (string) Session[SELECTED_ITEM];
            if (prevItemID != null)
            {
                var prevItem = (GridViewRow) (dataGrid).FindControl(prevItemID);
                if (prevItem != null)
                {
                    // REVIEW
                    SetItemColorRecursive(prevItem, Color.Black);
                    // prevButton.ForeColor = Color.Black;
                }
            }

            SetItemColorRecursive(item, Color.White);
            // Review: best practice for a naming container inside a user control.
            string id = item.UniqueID;
            int i = id.IndexOf((dataGrid).ID);
            id = id.Substring(i);
            i = id.IndexOf("$");
            id = id.Substring(i + 1);
            Session[SELECTED_ITEM] = id;
        }

        private void UpdateRules()
        {
            System.Configuration.Configuration config = OpenWebConfiguration(CurrentPath, true);
            var auth = (AuthorizationSection) config.GetSection("system.web/authorization");
            auth.Rules.Clear();
            ArrayList rules = Rules;
            foreach (AuthorizationRule rule in rules)
            {
                // // Response.Write("rule <br/>");
                auth.Rules.Add(rule);
            }

            // auth.IsModified = true;
            SaveConfig(config);
        }

        private void UpdateUpDownButtons()
        {
            int index = SelectedRule;
            moveDown.Enabled = (index < Rules.Count - ParentRuleCount - 1);
            moveUp.Enabled = (index > 0);
        }
    }
}